import { ParamsFrom } from "actionhero";
import { AuthenticatedAction } from "../classes/actions/authenticatedAction";
import { ActionPermission } from "../models/Permission";
import { Run } from "../models/Run";
import { APIData } from "../modules/apiData";
export declare class AppRefreshQueryRun extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly id: {
            readonly required: true;
        };
    };
    runWithinTransaction({ params, }: {
        params: ParamsFrom<AppRefreshQueryRun>;
    }): Promise<{
        valueUpdated: Boolean;
        appRefreshQuery: {
            id: string;
            appId: string;
            refreshQuery: string;
            value: string;
            locked: string;
            state: "ready" | "draft";
            lastChangedAt: number;
            lastConfirmedAt: number;
            recurringFrequency: number;
            createdAt: number;
            updatedAt: number;
        };
        runs: Run[];
    }>;
}
export declare class AppRefreshQueryCreate extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly appId: {
            readonly required: true;
        };
        readonly refreshQuery: {
            readonly required: false;
        };
        readonly recurringFrequency: {
            readonly required: false;
            readonly formatter: typeof APIData.ensureNumber;
        };
        readonly state: {
            readonly required: false;
        };
    };
    runWithinTransaction({ params, }: {
        params: ParamsFrom<AppRefreshQueryCreate>;
    }): Promise<{
        appRefreshQuery: {
            id: string;
            appId: string;
            refreshQuery: string;
            value: string;
            locked: string;
            state: "ready" | "draft";
            lastChangedAt: number;
            lastConfirmedAt: number;
            recurringFrequency: number;
            createdAt: number;
            updatedAt: number;
        };
    }>;
}
export declare class AppRefreshQueryEdit extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly id: {
            readonly required: true;
        };
        readonly appId: {
            readonly required: false;
        };
        readonly refreshQuery: {
            readonly required: false;
        };
        readonly recurringFrequency: {
            readonly required: false;
        };
        readonly state: {
            readonly required: false;
        };
    };
    runWithinTransaction({ params, }: {
        params: ParamsFrom<AppRefreshQueryEdit>;
    }): Promise<{
        appRefreshQuery: {
            id: string;
            appId: string;
            refreshQuery: string;
            value: string;
            locked: string;
            state: "ready" | "draft";
            lastChangedAt: number;
            lastConfirmedAt: number;
            recurringFrequency: number;
            createdAt: number;
            updatedAt: number;
        };
    }>;
}
export declare class AppRefreshQueryTest extends AuthenticatedAction {
    name: string;
    description: string;
    permission: ActionPermission;
    outputExample: {};
    inputs: {
        readonly id: {
            readonly required: true;
        };
        readonly refreshQuery: {
            readonly required: false;
        };
    };
    runWithinTransaction({ params, }: {
        params: ParamsFrom<AppRefreshQueryTest>;
    }): Promise<{
        test: {
            success: boolean;
            message: string;
            error: any;
        };
        appRefreshQuery: {
            id: string;
            appId: string;
            refreshQuery: string;
            value: string;
            locked: string;
            state: "ready" | "draft";
            lastChangedAt: number;
            lastConfirmedAt: number;
            recurringFrequency: number;
            createdAt: number;
            updatedAt: number;
        };
    }>;
}
export declare class AppRefreshQueryView extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly id: {
            readonly required: true;
        };
    };
    runWithinTransaction({ params, }: {
        params: ParamsFrom<AppRefreshQueryView>;
    }): Promise<{
        appRefreshQuery: {
            id: string;
            appId: string;
            refreshQuery: string;
            value: string;
            locked: string;
            state: "ready" | "draft";
            lastChangedAt: number;
            lastConfirmedAt: number;
            recurringFrequency: number;
            createdAt: number;
            updatedAt: number;
        };
    }>;
}
export declare class AppRefreshQueryDestroy extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly id: {
            readonly required: true;
        };
    };
    runWithinTransaction({ params, }: {
        params: ParamsFrom<AppRefreshQueryDestroy>;
    }): Promise<{
        success: boolean;
    }>;
}
