import { AuthenticatedAction } from "../classes/actions/authenticatedAction";
import { OptionHelper } from "../modules/optionHelper";
import { APIData } from "../modules/apiData";
import { ActionPermission } from "../models/Permission";
import { ParamsFrom } from "actionhero";
export declare class AppsList extends AuthenticatedAction {
    name: string;
    description: string;
    permission: ActionPermission;
    outputExample: {};
    inputs: {
        readonly limit: {
            readonly required: true;
            readonly default: 100;
            readonly formatter: typeof APIData.ensureNumber;
        };
        readonly offset: {
            readonly required: true;
            readonly default: 0;
            readonly formatter: typeof APIData.ensureNumber;
        };
        readonly state: {
            readonly required: false;
        };
        readonly order: {
            readonly required: false;
            readonly formatter: typeof APIData.ensureArray;
            readonly default: readonly [readonly ["name", "desc"], readonly ["createdAt", "desc"]];
        };
    };
    runWithinTransaction({ params }: {
        params: ParamsFrom<AppsList>;
    }): Promise<{
        total: number;
        apps: {
            id: string;
            name: string;
            icon: string;
            type: string;
            state: "ready" | "draft" | "deleted";
            locked: string;
            options: OptionHelper.SimpleOptions;
            provides: {
                source: boolean;
                destination: boolean;
            };
            pluginName: string;
            pluginApp: import("..").PluginApp;
            refreshQueryAvailable: boolean;
            createdAt: number;
            updatedAt: number;
            appRefreshQuery: {
                id: string;
                appId: string;
                refreshQuery: string;
                value: string;
                locked: string;
                state: "ready" | "draft";
                lastChangedAt: number;
                lastConfirmedAt: number;
                recurringFrequency: number;
                createdAt: number;
                updatedAt: number;
            };
        }[];
    }>;
}
export declare class AppOptions extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly id: {
            readonly required: true;
        };
    };
    runWithinTransaction({ params }: {
        params: ParamsFrom<AppOptions>;
    }): Promise<{
        options: {
            [optionName: string]: {
                type: import("..").PluginOptionType;
                options?: string[];
                descriptions?: string[];
            };
        };
        pluginOptions: import("..").AppOptionsOption[];
        environmentVariableOptions: string[];
    }>;
}
export declare class AppCreate extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly name: {
            readonly required: false;
        };
        readonly type: {
            readonly required: true;
        };
        readonly state: {
            readonly required: false;
        };
        readonly options: {
            readonly required: false;
            readonly formatter: typeof APIData.ensureObject;
        };
    };
    runWithinTransaction({ params }: {
        params: ParamsFrom<AppCreate>;
    }): Promise<{
        app: {
            id: string;
            name: string;
            icon: string;
            type: string;
            state: "ready" | "draft" | "deleted";
            locked: string;
            options: OptionHelper.SimpleOptions;
            provides: {
                source: boolean;
                destination: boolean;
            };
            pluginName: string;
            pluginApp: import("..").PluginApp;
            refreshQueryAvailable: boolean;
            createdAt: number;
            updatedAt: number;
            appRefreshQuery: {
                id: string;
                appId: string;
                refreshQuery: string;
                value: string;
                locked: string;
                state: "ready" | "draft";
                lastChangedAt: number;
                lastConfirmedAt: number;
                recurringFrequency: number;
                createdAt: number;
                updatedAt: number;
            };
        };
    }>;
}
export declare class AppEdit extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly id: {
            readonly required: true;
        };
        readonly name: {
            readonly required: false;
        };
        readonly type: {
            readonly required: false;
        };
        readonly state: {
            readonly required: false;
        };
        readonly options: {
            readonly required: false;
            readonly formatter: typeof APIData.ensureObject;
        };
    };
    runWithinTransaction({ params }: {
        params: ParamsFrom<AppEdit>;
    }): Promise<{
        app: {
            id: string;
            name: string;
            icon: string;
            type: string;
            state: "ready" | "draft" | "deleted";
            locked: string;
            options: OptionHelper.SimpleOptions;
            provides: {
                source: boolean;
                destination: boolean;
            };
            pluginName: string;
            pluginApp: import("..").PluginApp;
            refreshQueryAvailable: boolean;
            createdAt: number;
            updatedAt: number;
            appRefreshQuery: {
                id: string;
                appId: string;
                refreshQuery: string;
                value: string;
                locked: string;
                state: "ready" | "draft";
                lastChangedAt: number;
                lastConfirmedAt: number;
                recurringFrequency: number;
                createdAt: number;
                updatedAt: number;
            };
        };
    }>;
}
export declare class AppTest extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly id: {
            readonly required: true;
        };
        readonly options: {
            readonly required: false;
            readonly formatter: typeof APIData.ensureObject;
        };
    };
    runWithinTransaction({ params }: {
        params: ParamsFrom<AppTest>;
    }): Promise<{
        test: {
            success: boolean;
            message: string;
            error: any;
        };
        app: {
            id: string;
            name: string;
            icon: string;
            type: string;
            state: "ready" | "draft" | "deleted";
            locked: string;
            options: OptionHelper.SimpleOptions;
            provides: {
                source: boolean;
                destination: boolean;
            };
            pluginName: string;
            pluginApp: import("..").PluginApp;
            refreshQueryAvailable: boolean;
            createdAt: number;
            updatedAt: number;
            appRefreshQuery: {
                id: string;
                appId: string;
                refreshQuery: string;
                value: string;
                locked: string;
                state: "ready" | "draft";
                lastChangedAt: number;
                lastConfirmedAt: number;
                recurringFrequency: number;
                createdAt: number;
                updatedAt: number;
            };
        };
    }>;
}
export declare class AppView extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly id: {
            readonly required: true;
        };
    };
    runWithinTransaction({ params }: {
        params: ParamsFrom<AppView>;
    }): Promise<{
        app: {
            id: string;
            name: string;
            icon: string;
            type: string;
            state: "ready" | "draft" | "deleted";
            locked: string;
            options: OptionHelper.SimpleOptions;
            provides: {
                source: boolean;
                destination: boolean;
            };
            pluginName: string;
            pluginApp: import("..").PluginApp;
            refreshQueryAvailable: boolean;
            createdAt: number;
            updatedAt: number;
            appRefreshQuery: {
                id: string;
                appId: string;
                refreshQuery: string;
                value: string;
                locked: string;
                state: "ready" | "draft";
                lastChangedAt: number;
                lastConfirmedAt: number;
                recurringFrequency: number;
                createdAt: number;
                updatedAt: number;
            };
        };
    }>;
}
export declare class AppDestroy extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly id: {
            readonly required: true;
        };
    };
    runWithinTransaction({ params }: {
        params: ParamsFrom<AppDestroy>;
    }): Promise<{
        success: boolean;
    }>;
}
