import { ParamsFrom } from "actionhero";
import { AuthenticatedAction } from "../classes/actions/authenticatedAction";
import { OptionallyAuthenticatedAction } from "../classes/actions/optionallyAuthenticatedAction";
import { ConfigUser } from "../modules/configUser";
import { APIData } from "../modules/apiData";
import { ActionPermission } from "../models/Permission";
export declare class ConfigValidate extends AuthenticatedAction {
    name: string;
    description: string;
    permission: ActionPermission;
    inputs: {
        readonly local: {
            readonly required: true;
            readonly default: false;
            readonly formatter: typeof APIData.ensureBoolean;
        };
    };
    runWithinTransaction({ params, }: {
        params: ParamsFrom<ConfigValidate>;
    }): Promise<{
        exitCode: number;
        stdout: string;
        stderr: string;
        success: boolean;
    }>;
}
export declare class ConfigApply extends AuthenticatedAction {
    name: string;
    description: string;
    permission: ActionPermission;
    inputs: {
        readonly local: {
            readonly required: true;
            readonly default: false;
            readonly formatter: typeof APIData.ensureBoolean;
        };
    };
    runWithinTransaction({ params }: {
        params: ParamsFrom<ConfigApply>;
    }): Promise<{
        exitCode: number;
        stdout: string;
        stderr: string;
        success: boolean;
    }>;
}
export declare class ConfigUserCreate extends OptionallyAuthenticatedAction {
    name: string;
    description: string;
    permission: ActionPermission;
    inputs: {
        readonly email: {
            readonly required: true;
        };
        readonly company: {
            readonly required: true;
        };
        readonly subscribed: {
            readonly required: false;
            readonly formatter: typeof APIData.ensureBoolean;
        };
    };
    runWithinTransaction({ params, }: {
        params: ParamsFrom<ConfigUserCreate>;
    }): Promise<{
        user: ConfigUser.ConfigUserType;
    }>;
}
