import { ParamsFrom } from "actionhero";
import { AuthenticatedAction } from "../classes/actions/authenticatedAction";
import { ActionPermission } from "../models/Permission";
import { APIData } from "../modules/apiData";
export declare class ExportProcessorsList extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly destinationId: {
            readonly required: false;
        };
        readonly limit: {
            readonly required: true;
            readonly default: 100;
            readonly formatter: typeof APIData.ensureNumber;
        };
        readonly offset: {
            readonly required: true;
            readonly default: 0;
            readonly formatter: typeof APIData.ensureNumber;
        };
        readonly state: {
            readonly required: false;
        };
        readonly order: {
            readonly required: false;
            readonly default: readonly [readonly ["createdAt", "desc"]];
            readonly formatter: typeof APIData.ensureArray;
        };
    };
    runWithinTransaction({ params, }: {
        params: ParamsFrom<ExportProcessorsList>;
    }): Promise<{
        total: number;
        exportProcessors: {
            id: string;
            destination: {
                id: string;
                name: string;
                type: string;
                state: "ready" | "draft" | "deleted";
                locked: string;
                syncMode: "sync" | "additive" | "enrich";
                syncModes: {
                    key: "sync" | "additive" | "enrich";
                    displayName: string;
                    description: string;
                    operations: import("..").DestinationSyncOperations;
                }[];
                collection: "none" | "group" | "model";
                app: {
                    id: string;
                    name: string;
                    icon: string;
                    type: string;
                    state: "ready" | "draft" | "deleted";
                    locked: string;
                    options: import("../modules/optionHelper").OptionHelper.SimpleOptions;
                    provides: {
                        source: boolean;
                        destination: boolean;
                    };
                    pluginName: string;
                    pluginApp: import("..").PluginApp;
                    refreshQueryAvailable: boolean;
                    createdAt: number;
                    updatedAt: number;
                    appRefreshQuery: {
                        id: string;
                        appId: string;
                        refreshQuery: string;
                        value: string;
                        locked: string;
                        state: "ready" | "draft";
                        lastChangedAt: number;
                        lastConfirmedAt: number;
                        recurringFrequency: number;
                        createdAt: number;
                        updatedAt: number;
                    };
                };
                modelId: string;
                modelName: string;
                mapping: import("../modules/mappingHelper").MappingHelper.Mappings;
                options: import("../modules/optionHelper").OptionHelper.SimpleOptions;
                connection: import("..").PluginConnection;
                group: {
                    id: string;
                    name: string;
                    modelId: string;
                    modelName: string;
                    rules: import("..").GroupRuleWithKey[];
                    matchType: "all" | "any";
                    state: "ready" | "draft" | "deleted" | "initializing" | "updating";
                    locked: string;
                    recordsCount: number;
                    createdAt: number;
                    updatedAt: number;
                    nextCalculatedAt: number;
                    calculatedAt: number;
                };
                destinationGroupMemberships: import("../models/Destination").SimpleDestinationGroupMembership[];
                createdAt: number;
                updatedAt: number;
                exportTotals: {
                    failed: number;
                    canceled: number;
                    complete: number;
                    pending: number;
                    draft: number;
                    processing: number;
                };
            };
            destinationName: string;
            state: "failed" | "complete" | "pending";
            createdAt: number;
            processAt: number;
            startedAt: number;
            completedAt: number;
            remoteKey: string;
            exportsCount: number;
            retryCount: number;
            errorMessage: string;
            errorLevel: "error" | "info";
        }[];
    }>;
}
export declare class ExportProcessorView extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly id: {
            readonly required: true;
        };
    };
    runWithinTransaction({ params, }: {
        params: ParamsFrom<ExportProcessorView>;
    }): Promise<{
        exportProcessor: {
            id: string;
            destination: {
                id: string;
                name: string;
                type: string;
                state: "ready" | "draft" | "deleted";
                locked: string;
                syncMode: "sync" | "additive" | "enrich";
                syncModes: {
                    key: "sync" | "additive" | "enrich";
                    displayName: string;
                    description: string;
                    operations: import("..").DestinationSyncOperations;
                }[];
                collection: "none" | "group" | "model";
                app: {
                    id: string;
                    name: string;
                    icon: string;
                    type: string;
                    state: "ready" | "draft" | "deleted";
                    locked: string;
                    options: import("../modules/optionHelper").OptionHelper.SimpleOptions;
                    provides: {
                        source: boolean;
                        destination: boolean;
                    };
                    pluginName: string;
                    pluginApp: import("..").PluginApp;
                    refreshQueryAvailable: boolean;
                    createdAt: number;
                    updatedAt: number;
                    appRefreshQuery: {
                        id: string;
                        appId: string;
                        refreshQuery: string;
                        value: string;
                        locked: string;
                        state: "ready" | "draft";
                        lastChangedAt: number;
                        lastConfirmedAt: number;
                        recurringFrequency: number;
                        createdAt: number;
                        updatedAt: number;
                    };
                };
                modelId: string;
                modelName: string;
                mapping: import("../modules/mappingHelper").MappingHelper.Mappings;
                options: import("../modules/optionHelper").OptionHelper.SimpleOptions;
                connection: import("..").PluginConnection;
                group: {
                    id: string;
                    name: string;
                    modelId: string;
                    modelName: string;
                    rules: import("..").GroupRuleWithKey[];
                    matchType: "all" | "any";
                    state: "ready" | "draft" | "deleted" | "initializing" | "updating";
                    locked: string;
                    recordsCount: number;
                    createdAt: number;
                    updatedAt: number;
                    nextCalculatedAt: number;
                    calculatedAt: number;
                };
                destinationGroupMemberships: import("../models/Destination").SimpleDestinationGroupMembership[];
                createdAt: number;
                updatedAt: number;
                exportTotals: {
                    failed: number;
                    canceled: number;
                    complete: number;
                    pending: number;
                    draft: number;
                    processing: number;
                };
            };
            destinationName: string;
            state: "failed" | "complete" | "pending";
            createdAt: number;
            processAt: number;
            startedAt: number;
            completedAt: number;
            remoteKey: string;
            exportsCount: number;
            retryCount: number;
            errorMessage: string;
            errorLevel: "error" | "info";
        };
    }>;
}
