import { ParamsFrom } from "actionhero";
import { AuthenticatedAction } from "../classes/actions/authenticatedAction";
import { APIData } from "../modules/apiData";
import { ActionPermission } from "../models/Permission";
export declare class ExportsList extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly recordId: {
            readonly required: false;
        };
        readonly destinationId: {
            readonly required: false;
        };
        readonly exportProcessorId: {
            readonly required: false;
        };
        readonly limit: {
            readonly required: true;
            readonly default: 100;
            readonly formatter: typeof APIData.ensureNumber;
        };
        readonly offset: {
            readonly required: true;
            readonly default: 0;
            readonly formatter: typeof APIData.ensureNumber;
        };
        readonly state: {
            readonly required: false;
        };
        readonly order: {
            readonly required: false;
            readonly formatter: typeof APIData.ensureArray;
            readonly default: readonly [readonly ["createdAt", "desc"]];
        };
    };
    runWithinTransaction({ params }: {
        params: ParamsFrom<ExportsList>;
    }): Promise<{
        total: number;
        exports: {
            id: string;
            destination: {
                id: string;
                state: "ready" | "draft" | "deleted";
                name: string;
                groupId: string;
                modelId: string;
            };
            destinationName: string;
            recordId: string;
            modelId: string;
            exportProcessorId: string;
            state: "failed" | "canceled" | "complete" | "pending" | "draft" | "processing";
            force: boolean;
            createdAt: number;
            sendAt: number;
            startedAt: number;
            completedAt: number;
            retryCount: number;
            oldRecordProperties: import("../models/Export").ExportRecordProperties;
            newRecordProperties: import("../models/Export").ExportRecordProperties;
            oldGroups: string[];
            newGroups: string[];
            toDelete: boolean;
            hasChanges: boolean;
            errorMessage: string;
            errorLevel: "error" | "info";
        }[];
    }>;
}
export declare class ExportsTotals extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly recordId: {
            readonly required: false;
        };
        readonly destinationId: {
            readonly required: false;
        };
    };
    runWithinTransaction({ params, }: {
        params: ParamsFrom<ExportsTotals>;
    }): Promise<{
        totals: {
            failed: number;
            canceled: number;
            complete: number;
            pending: number;
            draft: number;
            processing: number;
        };
    }>;
}
export declare class ExportView extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly id: {
            readonly required: true;
        };
    };
    runWithinTransaction({ params }: {
        params: ParamsFrom<ExportView>;
    }): Promise<{
        export: {
            id: string;
            destination: {
                id: string;
                state: "ready" | "draft" | "deleted";
                name: string;
                groupId: string;
                modelId: string;
            };
            destinationName: string;
            recordId: string;
            modelId: string;
            exportProcessorId: string;
            state: "failed" | "canceled" | "complete" | "pending" | "draft" | "processing";
            force: boolean;
            createdAt: number;
            sendAt: number;
            startedAt: number;
            completedAt: number;
            retryCount: number;
            oldRecordProperties: import("../models/Export").ExportRecordProperties;
            newRecordProperties: import("../models/Export").ExportRecordProperties;
            oldGroups: string[];
            newGroups: string[];
            toDelete: boolean;
            hasChanges: boolean;
            errorMessage: string;
            errorLevel: "error" | "info";
        };
    }>;
}
export declare class ExportsRetryFailed extends AuthenticatedAction {
    name: string;
    description: string;
    permission: ActionPermission;
    inputs: {
        readonly destinationId: {
            readonly required: false;
        };
        readonly startTimestamp: {
            readonly required: true;
            readonly formatter: typeof APIData.ensureNumber;
        };
        readonly endTimestamp: {
            readonly required: true;
            readonly formatter: typeof APIData.ensureNumber;
        };
        readonly preview: {
            readonly required: false;
            readonly default: false;
            readonly formatter: typeof APIData.ensureBoolean;
        };
    };
    runWithinTransaction({ params, }: {
        params: ParamsFrom<ExportsRetryFailed>;
    }): Promise<{
        count: number;
    }>;
}
export declare class ExportRetryById extends AuthenticatedAction {
    name: string;
    description: string;
    permission: ActionPermission;
    inputs: {
        readonly exportId: {
            readonly required: true;
        };
    };
    runWithinTransaction({ params, }: {
        params: ParamsFrom<ExportRetryById>;
    }): Promise<{
        count: number;
    }>;
}
