import { AuthenticatedAction } from "../classes/actions/authenticatedAction";
import { APIData } from "../modules/apiData";
import { ActionPermission } from "../models/Permission";
import { ParamsFrom } from "actionhero";
export declare class ImportsList extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly creatorId: {
            readonly required: false;
        };
        readonly recordId: {
            readonly required: false;
        };
        readonly limit: {
            readonly required: true;
            readonly default: 100;
            readonly formatter: typeof APIData.ensureNumber;
        };
        readonly offset: {
            readonly required: true;
            readonly default: 0;
            readonly formatter: typeof APIData.ensureNumber;
        };
        readonly state: {
            readonly required: false;
        };
        readonly order: {
            readonly required: false;
            readonly formatter: typeof APIData.ensureArray;
            readonly default: readonly [readonly ["createdAt", "desc"]];
        };
    };
    runWithinTransaction({ params }: {
        params: ParamsFrom<ImportsList>;
    }): Promise<{
        total: number;
        imports: {
            state: "failed" | "complete" | "processing" | "importing" | "associating";
            id: string;
            creatorType: "group" | "run";
            creatorId: string;
            recordId: string;
            modelId: string;
            data: import("../models/Import").ImportData;
            createdAt: number;
            startedAt: number;
            recordAssociatedAt: number;
            importedAt: number;
            processedAt: number;
            createdRecord: boolean;
            errorMessage: string;
            errorMetadata: string;
        }[];
    }>;
}
export declare class ImportView extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly id: {
            readonly required: true;
        };
    };
    runWithinTransaction({ params }: {
        params: ParamsFrom<ImportView>;
    }): Promise<{
        import: {
            state: "failed" | "complete" | "processing" | "importing" | "associating";
            id: string;
            creatorType: "group" | "run";
            creatorId: string;
            recordId: string;
            modelId: string;
            data: import("../models/Import").ImportData;
            createdAt: number;
            startedAt: number;
            recordAssociatedAt: number;
            importedAt: number;
            processedAt: number;
            createdRecord: boolean;
            errorMessage: string;
            errorMetadata: string;
        };
    }>;
}
