"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NotificationView = exports.NotificationsList = void 0;
const authenticatedAction_1 = require("../classes/actions/authenticatedAction");
const Notification_1 = require("../models/Notification");
const sequelize_1 = require("sequelize");
const apiData_1 = require("../modules/apiData");
class NotificationsList extends authenticatedAction_1.AuthenticatedAction {
    constructor() {
        super(...arguments);
        this.name = "notifications:list";
        this.description = "list notification";
        this.outputExample = {};
        this.permission = { topic: "notification", mode: "read" };
        this.inputs = {
            limit: { required: true, default: 100, formatter: apiData_1.APIData.ensureNumber },
            offset: { required: true, default: 0, formatter: apiData_1.APIData.ensureNumber },
            order: {
                required: false,
                formatter: apiData_1.APIData.ensureArray,
                default: [["createdAt", "desc"]],
            },
        };
    }
    async runWithinTransaction({ params, }) {
        const search = {
            limit: params.limit,
            offset: params.offset,
            order: params.order,
            where: {},
        };
        const total = await Notification_1.Notification.count(search);
        const notifications = await Notification_1.Notification.findAll(search);
        const unreadCount = await Notification_1.Notification.count({
            where: { readAt: { [sequelize_1.Op.eq]: null } },
        });
        return {
            total,
            unreadCount,
            notifications: await Promise.all(notifications.map((notification) => notification.apiData())),
        };
    }
}
exports.NotificationsList = NotificationsList;
class NotificationView extends authenticatedAction_1.AuthenticatedAction {
    constructor() {
        super(...arguments);
        this.name = "notification:view";
        this.description = "view a notification";
        this.outputExample = {};
        this.permission = { topic: "notification", mode: "read" };
        this.inputs = {
            id: { required: true },
        };
    }
    async runWithinTransaction({ params, }) {
        const notification = await Notification_1.Notification.findById(params.id);
        await notification.update({ readAt: new Date() });
        return { notification: await notification.apiData() };
    }
}
exports.NotificationView = NotificationView;
