import { AuthenticatedAction } from "../classes/actions/authenticatedAction";
import { OptionallyAuthenticatedAction } from "../classes/actions/optionallyAuthenticatedAction";
import { ParamsFrom } from "actionhero";
import { APIData } from "../modules/apiData";
import { ActionPermission } from "../models/Permission";
export declare class PluginsList extends OptionallyAuthenticatedAction {
    name: string;
    description: string;
    permission: ActionPermission;
    inputs: {
        readonly includeInstalled: {
            readonly required: false;
            readonly formatter: typeof APIData.ensureBoolean;
        };
        readonly includeAvailable: {
            readonly required: false;
            readonly formatter: typeof APIData.ensureBoolean;
        };
        readonly includeVersions: {
            readonly required: false;
            readonly formatter: typeof APIData.ensureBoolean;
        };
    };
    runWithinTransaction({ params }: {
        params: ParamsFrom<PluginsList>;
    }): Promise<{
        plugins: import("../modules/plugins").PluginWithVersion[];
    }>;
}
export declare class PluginInstall extends AuthenticatedAction {
    name: string;
    description: string;
    permission: ActionPermission;
    inputs: {
        readonly plugin: {
            readonly required: true;
        };
        readonly restart: {
            readonly required: false;
            readonly default: false;
            readonly formatter: typeof APIData.ensureBoolean;
        };
    };
    runWithinTransaction({ params, }: {
        params: ParamsFrom<PluginInstall>;
    }): Promise<{
        success: boolean;
        checkIn?: number;
    }>;
}
export declare class PluginUninstall extends AuthenticatedAction {
    name: string;
    description: string;
    permission: ActionPermission;
    inputs: {
        readonly plugin: {
            readonly required: true;
        };
        readonly restart: {
            readonly required: false;
            readonly default: false;
            readonly formatter: typeof APIData.ensureBoolean;
        };
    };
    runWithinTransaction({ params, }: {
        params: ParamsFrom<PluginUninstall>;
    }): Promise<{
        success: boolean;
        checkIn?: undefined;
    } | {
        success: true;
        checkIn: number;
    }>;
}
