"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResetCache = exports.ResetData = exports.ResetCluster = void 0;
const authenticatedAction_1 = require("../classes/actions/authenticatedAction");
const reset_1 = require("../modules/reset");
class ResetCluster extends authenticatedAction_1.AuthenticatedAction {
    constructor() {
        super(...arguments);
        this.name = "reset:cluster";
        this.description = "Reset all data in the Grouparoo Cluster except for Teams and Team Members";
        this.outputExample = {};
        this.permission = { topic: "app", mode: "write" };
    }
    async runWithinTransaction({ session: { teamMember }, }) {
        const counts = await reset_1.Reset.cluster(teamMember.id);
        return { success: true, counts };
    }
}
exports.ResetCluster = ResetCluster;
class ResetData extends authenticatedAction_1.AuthenticatedAction {
    constructor() {
        super(...arguments);
        this.name = "reset:data";
        this.description = "Reset the imported and exported data in this cluster";
        this.outputExample = {};
        this.permission = { topic: "app", mode: "write" };
    }
    async runWithinTransaction({ session: { teamMember }, }) {
        await reset_1.Reset.data(teamMember.id);
        return { success: true };
    }
}
exports.ResetData = ResetData;
class ResetCache extends authenticatedAction_1.AuthenticatedAction {
    constructor() {
        super(...arguments);
        this.name = "reset:cache";
        this.description = "Reset the cache";
        this.outputExample = {};
        this.permission = { topic: "app", mode: "write" };
    }
    async runWithinTransaction({ session: { teamMember }, }) {
        await reset_1.Reset.cache(teamMember.id);
        return { success: true };
    }
}
exports.ResetCache = ResetCache;
