import { Action, Connection, ParamsFrom } from "actionhero";
import { AuthenticatedAction } from "../classes/actions/authenticatedAction";
import { CLSAction } from "../classes/actions/clsAction";
import { TeamMember } from "../models/TeamMember";
import { ActionPermission } from "../models/Permission";
export declare class SessionCreate extends CLSAction {
    name: string;
    description: string;
    inputs: {
        readonly email: {
            readonly required: true;
        };
        readonly password: {
            readonly required: false;
        };
        readonly requestId: {
            readonly required: false;
        };
    };
    outputExample: {};
    isWriteTransaction(): boolean;
    runWithinTransaction({ connection, params, }: {
        connection: Connection;
        params: ParamsFrom<SessionCreate>;
    }): Promise<{
        success: boolean;
        teamMember: {
            id: string;
            teamId: string;
            firstName: string;
            lastName: string;
            email: string;
            locked: string;
            lastLoginAt: Date;
            createdAt: number;
            updatedAt: number;
        };
        csrfToken: string;
    }>;
}
export declare class SessionView extends AuthenticatedAction {
    name: string;
    description: string;
    permission: ActionPermission;
    outputExample: {};
    runWithinTransaction({ connection, session: { teamMember }, }: {
        connection: Connection;
        session: {
            teamMember: TeamMember;
        };
    }): Promise<{
        teamMember: {
            id: string;
            teamId: string;
            firstName: string;
            lastName: string;
            email: string;
            locked: string;
            lastLoginAt: Date;
            createdAt: number;
            updatedAt: number;
        };
        csrfToken: string;
    }>;
}
export declare class SessionDestroy extends Action {
    name: string;
    description: string;
    outputExample: {};
    run({ connection }: {
        connection: Connection;
    }): Promise<{
        success: boolean;
    }>;
}
