import { AuthenticatedAction } from "../classes/actions/authenticatedAction";
import { OptionallyAuthenticatedAction } from "../classes/actions/optionallyAuthenticatedAction";
import { APIData } from "../modules/apiData";
import { ActionPermission } from "../models/Permission";
import { ParamsFrom } from "actionhero";
export declare class SettingsList extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly order: {
            readonly required: false;
            readonly formatter: typeof APIData.ensureArray;
            readonly default: readonly [readonly ["pluginName", "desc"], readonly ["title", "desc"], readonly ["key", "desc"]];
        };
    };
    runWithinTransaction({ params }: {
        params: ParamsFrom<SettingsList>;
    }): Promise<{
        settings: {
            id: string;
            pluginName: string;
            key: string;
            title: string;
            value: string;
            type: "string" | "number" | "boolean";
            variant: string;
            defaultValue: string;
            description: string;
            locked: string;
            createdAt: number;
            updatedAt: number;
        }[];
    }>;
}
export declare class SettingCoreClusterName extends OptionallyAuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    runWithinTransaction(): Promise<{
        setting: {
            id: string;
            pluginName: string;
            key: string;
            title: string;
            value: string;
            type: "string" | "number" | "boolean";
            variant: string;
            defaultValue: string;
            description: string;
            locked: string;
            createdAt: number;
            updatedAt: number;
        };
    }>;
}
export declare class SettingEdit extends AuthenticatedAction {
    name: string;
    description: string;
    outputExample: {};
    permission: ActionPermission;
    inputs: {
        readonly id: {
            readonly required: true;
        };
        readonly value: {
            readonly required: true;
        };
    };
    runWithinTransaction({ params }: {
        params: ParamsFrom<SettingEdit>;
    }): Promise<{
        setting: {
            id: string;
            pluginName: string;
            key: string;
            title: string;
            value: string;
            type: "string" | "number" | "boolean";
            variant: string;
            defaultValue: string;
            description: string;
            locked: string;
            createdAt: number;
            updatedAt: number;
        };
    }>;
}
