"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ResetCLI = void 0;
const cli_1 = require("../modules/cli");
const actionhero_1 = require("actionhero");
const reset_1 = require("../modules/reset");
const cls_1 = require("../modules/cls");
class ResetCLI extends actionhero_1.CLI {
    constructor() {
        super(...arguments);
        this.name = "reset <mode>";
        this.description = "Reset part of this Cluster. Mode can be `cluster` (all data), `data` (imported data) and `cache`";
        this.inputs = {
            confirm: {
                required: false,
                default: false,
                flag: true,
                letter: "c",
                description: "Confirm the command",
            },
        };
        this.preInitialize = () => {
            cli_1.GrouparooCLI.setGrouparooRunMode(this);
            cli_1.GrouparooCLI.setNextDevelopmentMode();
        };
    }
    async run({ params, }) {
        const [mode] = params._arguments || [];
        if (mode)
            params.mode = mode;
        cli_1.GrouparooCLI.logCLI(this.name.replace(" <mode>", params.mode ? " " + params.mode : ""), false);
        if (!params.confirm) {
            return cli_1.GrouparooCLI.logger.fatal(`You need to --confirm this command`);
        }
        const callerId = `cli:reset`;
        if (params.mode === "cluster") {
            await cls_1.CLS.wrap(async () => await reset_1.Reset.cluster(callerId));
        }
        else if (params.mode === "data") {
            await cls_1.CLS.wrap(async () => await reset_1.Reset.data(callerId));
            await cls_1.CLS.wrap(async () => await reset_1.Reset.resetHighWatermarks());
        }
        else if (params.mode === "cache") {
            await cls_1.CLS.wrap(async () => await reset_1.Reset.cache(callerId));
        }
        else {
            return cli_1.GrouparooCLI.logger.fatal("mode not found");
        }
        cli_1.GrouparooCLI.logger.log(`✅ Success!`);
        return true;
    }
}
exports.ResetCLI = ResetCLI;
