import Moment from "moment";
import { CLI, ParamsFrom } from "actionhero";
import { APIData } from "../modules/apiData";
import { Destination } from "../models/Destination";
export declare class RetryExportsCLI extends CLI {
    name: string;
    description: string;
    inputs: {
        readonly start: {
            readonly required: false;
            readonly requiredValue: true;
            readonly formatter: typeof APIData.ensureDate;
            readonly description: "Search for failed Exports created on or after this timestamp.";
        };
        readonly startAgo: {
            readonly required: false;
            readonly requiredValue: true;
            readonly formatter: typeof APIData.ensureNumber;
            readonly description: "Search for failed Exports created on or after a certain number of seconds ago. See --startAgoUnit to choose a different time unit.";
        };
        readonly startAgoUnit: {
            readonly required: false;
            readonly requiredValue: true;
            readonly default: "second";
            readonly description: "Unit for --startAgo (e.g. seconds, minutes, hours, days...)";
            readonly formatter: (val: string) => Moment.unitOfTime.Base;
            readonly validator: (val: string) => void;
        };
        readonly end: {
            readonly required: false;
            readonly requiredValue: true;
            readonly formatter: typeof APIData.ensureDate;
            readonly description: "Search for failed Exports created on or before this timestamp. Defaults to the current time.";
        };
        readonly preview: {
            readonly default: false;
            readonly formatter: (val: any) => boolean;
            readonly description: "When set, will not make any changes and only outputs the number of Exports that would be retried.";
            readonly flag: true;
            readonly letter: "p";
        };
        readonly destinationIds: {
            readonly required: false;
            readonly requiredValue: true;
            readonly description: "Only retry Exports for specific Destinations. Defaults to all Destinations.";
            readonly letter: "d";
            readonly variadic: true;
        };
    };
    constructor();
    preInitialize: () => void;
    run({ params }: {
        params: Partial<ParamsFrom<RetryExportsCLI>>;
    }): Promise<boolean>;
    getDestinations(destinationIds?: string[]): Promise<Destination[]>;
}
