"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommonModel = void 0;
const sequelize_typescript_1 = require("sequelize-typescript");
const validator_1 = __importDefault(require("validator"));
const uuid = __importStar(require("uuid"));
const sequelize_1 = require("sequelize");
const actionhero_1 = require("actionhero");
const errors_1 = require("../modules/errors");
// This tighter type is not yet in use because it would require all places where we make a new instance to typecheck.
//  For example, an action that takes state:string would need to coerce that to state: typeof App['state'].
//  That is out of scope for now.
//
// export abstract class CommonModel<T> extends Model<T, NonAbstract<Columns<T>>> {
class CommonModel extends sequelize_typescript_1.Model {
    static generateId(instance) {
        if (!instance.id)
            instance.id = `${instance.idPrefix()}_${uuid.v4()}`;
    }
    static generateIds(instances) {
        instances.forEach((instance) => this.generateId(instance));
    }
    static validateId(instance) {
        const id = instance.id;
        let failing = false;
        if (id.length > 191)
            failing = true;
        if (!/^[A-Za-z0-9-_]+$/.test(id))
            failing = true; // only allow letters, numbers, hyphen and underscore
        if (failing) {
            throw new Error(`invalid id: \`${id}\` - ids must be less than 191 characters and not contain spaces or special characters`);
        }
    }
    static validateIds(instances) {
        instances.forEach((instance) => this.validateId(instance));
    }
    async touch() {
        this.changed("updatedAt", true);
        return this.save({ hooks: false });
    }
    idIsDefault() {
        const uuid = this.id.split("_").pop();
        return this.id.startsWith(`${this.idPrefix()}_`) && validator_1.default.isUUID(uuid);
    }
    // --- Class Methods --- //
    /**
     * Find an instance of this class, regardless of scope.
     * Throw if the instance cannot be found.
     */
    static async findById(id) {
        var _a, _b;
        const instance = await this.scope(null).findOne({ where: { id } });
        const modelName = (_b = (_a = String(this).match(/class\s(.+)\sextends/)) === null || _a === void 0 ? void 0 : _a[1]) !== null && _b !== void 0 ? _b : "item"; // TODO: It looks like at this point `this` is the stringified representation of the constructor function, literally "class Group extends CommonModel..."
        if (!instance)
            throw new Error(`cannot find ${modelName} ${id}`);
        return instance;
    }
    /**
     * Update many instances at once, never exceeding a set batch size for
     */
    static async updateAllInBatches(instances, values) {
        const max = actionhero_1.config.batchSize.internalWrite;
        const ids = instances.map((i) => i.id);
        const queue = [...ids];
        while (queue.length > 0) {
            await this.update(values, {
                where: { id: { [sequelize_1.Op.in]: queue.splice(0, max) } },
            });
        }
    }
    /**
     * Ensures there isn't a duplicate version of this instance based on name or key.
     */
    static async ensureUnique(instance) {
        function getUniqueIdentifier(instance) {
            if (instance === null || instance === void 0 ? void 0 : instance.uniqueIdentifier) {
                return instance.uniqueIdentifier;
            }
            else {
                return (instance === null || instance === void 0 ? void 0 : instance.name) !== undefined
                    ? ["name"]
                    : (instance === null || instance === void 0 ? void 0 : instance.key) !== undefined
                        ? ["key"]
                        : undefined;
            }
        }
        const instanceUniqueIdentifiers = getUniqueIdentifier(instance);
        if (!instanceUniqueIdentifiers) {
            return;
        }
        const whereOpts = {
            id: { [sequelize_1.Op.ne]: instance.id },
        };
        instanceUniqueIdentifiers.forEach((identifier) => {
            if (typeof instance[identifier] === "string") {
                whereOpts[identifier] = (0, sequelize_1.where)((0, sequelize_1.fn)("LOWER", (0, sequelize_1.col)(String(identifier))), String(instance[identifier]).toLowerCase());
            }
            else {
                whereOpts[identifier] = instance[identifier];
            }
        });
        if (instance.state) {
            whereOpts.state = { [sequelize_1.Op.notIn]: ["draft", "deleted"] };
        }
        const duplicates = await this.count({
            where: whereOpts,
        });
        if (duplicates > 0) {
            // The unique key defaults to anything defined on the class, then name, then key.
            throw new errors_1.Errors.UniqueError(instanceUniqueIdentifiers
                .map((id) => `${id} "${instance[id]}" is already in use`)
                .join(", ") + ` in table ${this.toString().split(" ")[1]}`, this.toString().split(" ")[1], instanceUniqueIdentifiers, whereOpts);
        }
    }
}
__decorate([
    (0, sequelize_typescript_1.Length)({ min: 1, max: 191 }),
    (0, sequelize_typescript_1.Column)({ primaryKey: true }),
    __metadata("design:type", String)
], CommonModel.prototype, "id", void 0);
__decorate([
    sequelize_typescript_1.CreatedAt,
    __metadata("design:type", Date)
], CommonModel.prototype, "createdAt", void 0);
__decorate([
    sequelize_typescript_1.UpdatedAt,
    __metadata("design:type", Date)
], CommonModel.prototype, "updatedAt", void 0);
__decorate([
    sequelize_typescript_1.BeforeCreate,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [CommonModel]),
    __metadata("design:returntype", void 0)
], CommonModel, "generateId", null);
__decorate([
    sequelize_typescript_1.BeforeBulkCreate,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", void 0)
], CommonModel, "generateIds", null);
__decorate([
    sequelize_typescript_1.BeforeCreate,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [CommonModel]),
    __metadata("design:returntype", void 0)
], CommonModel, "validateId", null);
__decorate([
    sequelize_typescript_1.BeforeBulkCreate,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Array]),
    __metadata("design:returntype", void 0)
], CommonModel, "validateIds", null);
__decorate([
    sequelize_typescript_1.BeforeSave,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], CommonModel, "ensureUnique", null);
exports.CommonModel = CommonModel;
