"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.test = exports.DEFAULT = void 0;
const winston = __importStar(require("winston"));
const namespace = "logger";
exports.DEFAULT = {
    [namespace]: (config) => {
        const loggers = [];
        loggers.push(buildConsoleLogger(process.env.GROUPAROO_LOG_LEVEL));
        config.general.paths.log.forEach((p) => {
            loggers.push(buildFileLogger(p, process.env.GROUPAROO_LOG_LEVEL));
        });
        return {
            loggers,
            maxLogStringLength: 100,
            maxLogArrayLength: 10, // the maximum number of items in an array to log before collapsing into one message
        };
    },
};
exports.test = {
    [namespace]: (config) => {
        var _a;
        const loggers = [];
        loggers.push(buildConsoleLogger((_a = process.env.GROUPAROO_LOG_LEVEL) !== null && _a !== void 0 ? _a : "crit"));
        config.general.paths.log.forEach((path) => {
            loggers.push(buildFileLogger(path, "debug", 1));
        });
        return { loggers };
    },
};
// helpers for building the winston loggers
function buildConsoleLogger(level = "info") {
    const formats = [];
    if (!disableColor())
        formats.push(winston.format.colorize());
    if (!disableTimestamps())
        formats.push(winston.format.timestamp());
    formats.push(winston.format.printf((info) => {
        return `${info.timestamp && !disableTimestamps() ? `${info.timestamp} - ` : ""}${info.level}: ${info.message} ${stringifyExtraMessagePropertiesForConsole(info)}`;
    }));
    return function () {
        return winston.createLogger({
            format: winston.format.combine(...formats),
            level,
            levels: winston.config.syslog.levels,
            transports: [new winston.transports.Console()],
        });
    };
}
function buildFileLogger(path, level = "info", maxFiles) {
    return function (config) {
        const filename = `${path}/${config.process.id}-${config.process.env}.log`;
        return winston.createLogger({
            format: winston.format.combine(winston.format.timestamp(), winston.format.json()),
            level,
            levels: winston.config.syslog.levels,
            transports: [
                new winston.transports.File({
                    filename,
                    maxFiles,
                }),
            ],
        });
    };
}
function stringifyExtraMessagePropertiesForConsole(info) {
    const skippedProperties = ["message", "timestamp", "level"];
    let response = "";
    for (const key in info) {
        const value = info[key];
        if (skippedProperties.includes(key)) {
            continue;
        }
        if (value === undefined || value === null || value === "") {
            continue;
        }
        response += `${key}=${value} `;
    }
    return response;
}
const disableColor = () => { var _a; return ((_a = process.env.GROUPAROO_LOGS_STDOUT_DISABLE_COLOR) === null || _a === void 0 ? void 0 : _a.toLowerCase()) === "true"; };
const disableTimestamps = () => { var _a; return ((_a = process.env.GROUPAROO_LOGS_STDOUT_DISABLE_TIMESTAMP) === null || _a === void 0 ? void 0 : _a.toLowerCase()) === "true"; };
