"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TeamPermissions = void 0;
const actionhero_1 = require("actionhero");
const Team_1 = require("../models/Team");
const locks_1 = require("../modules/locks");
const clsInitializer_1 = require("../classes/initializers/clsInitializer");
class TeamPermissions extends clsInitializer_1.CLSInitializer {
    constructor() {
        super();
        this.name = "teamPermissions";
        this.startPriority = 2;
    }
    async updateTeamPermissions(team) {
        const { releaseLock } = await (0, locks_1.waitForLock)(`team:${team.id}:updateTeamPermissions`);
        try {
            const permissionsWithStatus = await Team_1.Team.buildPermissions(team);
            permissionsWithStatus.map(({ isNew, permission }) => {
                (0, actionhero_1.log)(`${isNew ? "Created new" : "Checked"} permission record for team ${team.name} @ ${permission.topic}`, isNew ? "notice" : "debug", { read: permission.read, write: permission.write });
            });
        }
        finally {
            releaseLock();
        }
    }
    async initializeWithinTransaction() { }
    async startWithinTransaction() {
        const teams = await Team_1.Team.findAll();
        for (const i in teams) {
            await this.updateTeamPermissions(teams[i]);
        }
    }
    async stopWithinTransaction() { }
}
exports.TeamPermissions = TeamPermissions;
