"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var AppRefreshQuery_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppRefreshQuery = void 0;
const sequelize_typescript_1 = require("sequelize-typescript");
const stateMachine_1 = require("../modules/stateMachine");
const lockableHelper_1 = require("../modules/lockableHelper");
const apiData_1 = require("../modules/apiData");
const appRefreshQuery_1 = require("../modules/ops/appRefreshQuery");
const App_1 = require("./App");
const cls_1 = require("../modules/cls");
const commonModel_1 = require("../classes/commonModel");
const STATES = ["draft", "ready"];
const STATE_TRANSITIONS = [
    { from: "draft", to: "ready", checks: [] },
];
let AppRefreshQuery = AppRefreshQuery_1 = class AppRefreshQuery extends commonModel_1.CommonModel {
    idPrefix() {
        return "adr";
    }
    async apiData() {
        return {
            id: this.id,
            appId: this.appId,
            refreshQuery: this.refreshQuery,
            value: this.value,
            locked: this.locked,
            state: this.state,
            lastChangedAt: apiData_1.APIData.formatDate(this.lastChangedAt),
            lastConfirmedAt: apiData_1.APIData.formatDate(this.lastConfirmedAt),
            recurringFrequency: this.recurringFrequency,
            createdAt: apiData_1.APIData.formatDate(this.createdAt),
            updatedAt: apiData_1.APIData.formatDate(this.updatedAt),
        };
    }
    async test(refreshQuery) {
        return await appRefreshQuery_1.AppRefreshQueryOps.test(this, refreshQuery);
    }
    async query() {
        return await appRefreshQuery_1.AppRefreshQueryOps.runAppQuery(this);
    }
    async shouldRun() {
        return appRefreshQuery_1.AppRefreshQueryOps.shouldRun(this);
    }
    async triggerSchedules(stopRuns) {
        return await appRefreshQuery_1.AppRefreshQueryOps.triggerSchedules(this, stopRuns);
    }
    // --- Class Methods --- //
    static async ensureOnePerApp(instance) {
        const existingCount = await AppRefreshQuery_1.scope(null).count({
            where: {
                appId: instance.appId,
            },
        });
        if (existingCount > 0) {
            throw new Error(`app ${instance.appId} already has an app refresh query`);
        }
    }
    static async noUpdateIfLocked(instance) {
        await lockableHelper_1.LockableHelper.beforeSave(instance, [
            "state",
            "value",
            "lastChangedAt",
            "lastConfirmedAt",
        ]);
    }
    static async updateState(instance) {
        await stateMachine_1.StateMachine.transition(instance, STATE_TRANSITIONS);
    }
    static async appQuery(instance) {
        //run the query to make sure it's valid
        if (instance.state === "ready") {
            await appRefreshQuery_1.AppRefreshQueryOps.runAppQuery(instance);
        }
        //if the query has been changed, reset it and enqueue run.
        if (instance.changed("refreshQuery")) {
            instance.lastConfirmedAt = null;
            instance.lastChangedAt = null;
            instance.value = null;
            await cls_1.CLS.enqueueTask("appRefreshQuery:run", {
                appRefreshQueryId: instance.id,
            });
        }
    }
    static async checkRecurringFrequency(instance) {
        // we cannot use the @Min validator as null is also allowed
        if (instance.recurringFrequency) {
            if (instance.recurringFrequency < 1000 * 60) {
                throw new Error("recurring frequency is required to be one minute or greater");
            }
        }
    }
    static async noDestroyIfLocked(instance) {
        await lockableHelper_1.LockableHelper.beforeDestroy(instance);
    }
};
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    (0, sequelize_typescript_1.ForeignKey)(() => App_1.App),
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], AppRefreshQuery.prototype, "appId", void 0);
__decorate([
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], AppRefreshQuery.prototype, "refreshQuery", void 0);
__decorate([
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], AppRefreshQuery.prototype, "value", void 0);
__decorate([
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], AppRefreshQuery.prototype, "locked", void 0);
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    (0, sequelize_typescript_1.Default)("draft"),
    (0, sequelize_typescript_1.Column)(sequelize_typescript_1.DataType.ENUM(...STATES)),
    __metadata("design:type", Object)
], AppRefreshQuery.prototype, "state", void 0);
__decorate([
    sequelize_typescript_1.Column,
    __metadata("design:type", Date)
], AppRefreshQuery.prototype, "lastChangedAt", void 0);
__decorate([
    sequelize_typescript_1.Column,
    __metadata("design:type", Date)
], AppRefreshQuery.prototype, "lastConfirmedAt", void 0);
__decorate([
    sequelize_typescript_1.Column,
    __metadata("design:type", Number)
], AppRefreshQuery.prototype, "recurringFrequency", void 0);
__decorate([
    (0, sequelize_typescript_1.BelongsTo)(() => App_1.App),
    __metadata("design:type", App_1.App)
], AppRefreshQuery.prototype, "app", void 0);
__decorate([
    sequelize_typescript_1.BeforeCreate,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [AppRefreshQuery]),
    __metadata("design:returntype", Promise)
], AppRefreshQuery, "ensureOnePerApp", null);
__decorate([
    sequelize_typescript_1.BeforeSave,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [AppRefreshQuery]),
    __metadata("design:returntype", Promise)
], AppRefreshQuery, "noUpdateIfLocked", null);
__decorate([
    sequelize_typescript_1.BeforeSave,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [AppRefreshQuery]),
    __metadata("design:returntype", Promise)
], AppRefreshQuery, "updateState", null);
__decorate([
    sequelize_typescript_1.BeforeSave,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [AppRefreshQuery]),
    __metadata("design:returntype", Promise)
], AppRefreshQuery, "appQuery", null);
__decorate([
    sequelize_typescript_1.BeforeSave,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [AppRefreshQuery]),
    __metadata("design:returntype", Promise)
], AppRefreshQuery, "checkRecurringFrequency", null);
__decorate([
    sequelize_typescript_1.BeforeDestroy,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [AppRefreshQuery]),
    __metadata("design:returntype", Promise)
], AppRefreshQuery, "noDestroyIfLocked", null);
AppRefreshQuery = AppRefreshQuery_1 = __decorate([
    (0, sequelize_typescript_1.DefaultScope)(() => ({
        where: { state: "ready" },
    })),
    (0, sequelize_typescript_1.Table)({ tableName: "appRefreshQueries", paranoid: false })
], AppRefreshQuery);
exports.AppRefreshQuery = AppRefreshQuery;
