"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Option = exports.OptionTypes = void 0;
const sequelize_typescript_1 = require("sequelize-typescript");
const App_1 = require("./App");
const Source_1 = require("./Source");
const Destination_1 = require("./Destination");
const apiData_1 = require("../modules/apiData");
const commonModel_1 = require("../classes/commonModel");
exports.OptionTypes = ["boolean", "string", "number"];
let Option = class Option extends commonModel_1.CommonModel {
    constructor() {
        super(...arguments);
        this.uniqueIdentifier = ["key", "ownerId", "ownerType"];
    }
    idPrefix() {
        return "opt";
    }
    async apiData() {
        return {
            id: this.id,
            ownerId: this.ownerId,
            ownerType: this.ownerType,
            key: this.key,
            value: this.typedValue(),
            createdAt: apiData_1.APIData.formatDate(this.createdAt),
            updatedAt: apiData_1.APIData.formatDate(this.updatedAt),
        };
    }
    typedValue() {
        if (this.type === "string")
            return this.value.toString();
        else if (this.type === "number")
            return parseFloat(this.value);
        else if (this.type === "boolean") {
            const stringValue = this.value.toString().toLowerCase().trim();
            return stringValue === "true" || stringValue === "1";
        }
        throw new Error(`cannot format option type ${this.type}`);
    }
    // --- Class Methods --- //
    static async validateType(instance) {
        if (!exports.OptionTypes.includes(instance.type)) {
            throw new Error(`${instance.type} is not a valid type for an Option (only ${exports.OptionTypes.join(", ")} allowed)`);
        }
    }
};
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    (0, sequelize_typescript_1.ForeignKey)(() => App_1.App),
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], Option.prototype, "ownerId", void 0);
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], Option.prototype, "ownerType", void 0);
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], Option.prototype, "key", void 0);
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], Option.prototype, "value", void 0);
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    (0, sequelize_typescript_1.Column)(sequelize_typescript_1.DataType.ENUM(...exports.OptionTypes)),
    __metadata("design:type", Object)
], Option.prototype, "type", void 0);
__decorate([
    (0, sequelize_typescript_1.BelongsTo)(() => App_1.App, "ownerId"),
    __metadata("design:type", App_1.App)
], Option.prototype, "app", void 0);
__decorate([
    (0, sequelize_typescript_1.BelongsTo)(() => Source_1.Source, "ownerId"),
    __metadata("design:type", Source_1.Source)
], Option.prototype, "source", void 0);
__decorate([
    (0, sequelize_typescript_1.BelongsTo)(() => Destination_1.Destination, "ownerId"),
    __metadata("design:type", Destination_1.Destination)
], Option.prototype, "destination", void 0);
__decorate([
    sequelize_typescript_1.BeforeSave,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Option]),
    __metadata("design:returntype", Promise)
], Option, "validateType", null);
Option = __decorate([
    (0, sequelize_typescript_1.Table)({ tableName: "options", paranoid: false })
], Option);
exports.Option = Option;
