"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Setting = exports.settingTypes = void 0;
const sequelize_typescript_1 = require("sequelize-typescript");
const configWriter_1 = require("../modules/configWriter");
const lockableHelper_1 = require("../modules/lockableHelper");
const apiData_1 = require("../modules/apiData");
const commonModel_1 = require("../classes/commonModel");
exports.settingTypes = ["string", "number", "boolean"];
let Setting = class Setting extends commonModel_1.CommonModel {
    constructor() {
        super(...arguments);
        this.uniqueIdentifier = ["key", "pluginName"];
    }
    idPrefix() {
        return "set";
    }
    get value() {
        const rawValue = this.getDataValue("value").toString();
        switch (this.type) {
            case "boolean":
                // SQLite returns booleans as numbers
                const booleanValue = rawValue === "true" || rawValue === "1";
                return booleanValue ? "true" : "false";
            default:
                return rawValue;
        }
    }
    set value(data) {
        this.setDataValue("value", data);
    }
    async apiData() {
        return {
            id: this.id,
            pluginName: this.pluginName,
            key: this.key,
            title: this.title,
            value: this.value,
            type: this.type,
            variant: this.variant,
            defaultValue: this.defaultValue,
            description: this.description,
            locked: this.locked,
            createdAt: apiData_1.APIData.formatDate(this.createdAt),
            updatedAt: apiData_1.APIData.formatDate(this.updatedAt),
        };
    }
    getConfigId() {
        return this.idIsDefault() ? configWriter_1.ConfigWriter.generateId(this.key) : this.id;
    }
    async getConfigObject() {
        const { pluginName, key, value } = this;
        return {
            id: this.getConfigId(),
            class: "Setting",
            pluginName,
            key,
            value,
        };
    }
    // --- Class Methods --- //
    static async validateType(instance) {
        if (!exports.settingTypes.includes(instance.type)) {
            throw new Error(`${instance.type} is not a valid Setting type`);
        }
    }
    static validateValue(instance) {
        switch (instance.type) {
            case "boolean": {
                // already coerced by setter/getter
                break;
            }
            case "string": {
                // anything is fine
                break;
            }
            case "number": {
                const numberValue = parseFloat(instance.value);
                if (isNaN(numberValue)) {
                    throw new Error(`${instance.value} is not a number`);
                }
                if (numberValue < 0) {
                    throw new Error(`setting values cannot have negative numbers`);
                }
                break;
            }
        }
    }
    static async noUpdateIfLocked(instance) {
        await lockableHelper_1.LockableHelper.beforeSave(instance);
    }
    static async noDestroyIfLocked(instance) {
        await lockableHelper_1.LockableHelper.beforeDestroy(instance);
    }
};
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], Setting.prototype, "pluginName", void 0);
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], Setting.prototype, "key", void 0);
__decorate([
    sequelize_typescript_1.Column,
    __metadata("design:type", String),
    __metadata("design:paramtypes", [Object])
], Setting.prototype, "value", null);
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], Setting.prototype, "defaultValue", void 0);
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    (0, sequelize_typescript_1.Column)(sequelize_typescript_1.DataType.ENUM(...exports.settingTypes)),
    __metadata("design:type", Object)
], Setting.prototype, "type", void 0);
__decorate([
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], Setting.prototype, "locked", void 0);
__decorate([
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], Setting.prototype, "title", void 0);
__decorate([
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], Setting.prototype, "description", void 0);
__decorate([
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], Setting.prototype, "variant", void 0);
__decorate([
    sequelize_typescript_1.BeforeSave,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Setting]),
    __metadata("design:returntype", Promise)
], Setting, "validateType", null);
__decorate([
    sequelize_typescript_1.BeforeSave,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Setting]),
    __metadata("design:returntype", void 0)
], Setting, "validateValue", null);
__decorate([
    sequelize_typescript_1.BeforeSave,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Setting]),
    __metadata("design:returntype", Promise)
], Setting, "noUpdateIfLocked", null);
__decorate([
    sequelize_typescript_1.BeforeDestroy,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Setting]),
    __metadata("design:returntype", Promise)
], Setting, "noDestroyIfLocked", null);
Setting = __decorate([
    (0, sequelize_typescript_1.Table)({ tableName: "settings", paranoid: false })
], Setting);
exports.Setting = Setting;
