"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SetupStep = void 0;
const sequelize_typescript_1 = require("sequelize-typescript");
const setupSteps_1 = require("../modules/ops/setupSteps");
const apiData_1 = require("../modules/apiData");
const commonModel_1 = require("../classes/commonModel");
let SetupStep = class SetupStep extends commonModel_1.CommonModel {
    idPrefix() {
        return "sus";
    }
    async apiData(modelId) {
        const ssd = this.getSetupStepDescription(modelId);
        const title = this.getTitle(ssd);
        const description = this.getDescription(ssd);
        const href = this.getHref(ssd);
        const cta = this.getCta(ssd);
        const helpLink = this.getHelpLink(ssd);
        const showCtaOnCommunity = this.getShowCtaOnCommunity(ssd);
        const outcome = await this.getOutcome(ssd);
        const disabled = await this.getDisabled(ssd);
        return {
            id: this.id,
            position: this.position,
            key: this.key,
            title,
            description,
            href,
            cta,
            helpLink,
            showCtaOnCommunity,
            outcome,
            disabled,
            skipped: this.skipped,
            complete: this.complete,
            createdAt: apiData_1.APIData.formatDate(this.createdAt),
            updatedAt: apiData_1.APIData.formatDate(this.updatedAt),
        };
    }
    getTitle(ssd, modelId) {
        if (!ssd)
            ssd = this.getSetupStepDescription(modelId);
        return ssd.title;
    }
    getDescription(ssd, modelId) {
        if (!ssd)
            ssd = this.getSetupStepDescription(modelId);
        return ssd.description;
    }
    getHref(ssd, modelId) {
        if (!ssd)
            ssd = this.getSetupStepDescription(modelId);
        return ssd.href;
    }
    getCta(ssd, modelId) {
        if (!ssd)
            ssd = this.getSetupStepDescription(modelId);
        return ssd.cta;
    }
    getHelpLink(ssd, modelId) {
        if (!ssd)
            ssd = this.getSetupStepDescription(modelId);
        return ssd.helpLink;
    }
    getShowCtaOnCommunity(ssd, modelId) {
        if (!ssd)
            ssd = this.getSetupStepDescription(modelId);
        return ssd.showCtaOnCommunity || false;
    }
    async performCheck(ssd, modelId) {
        if (this.complete)
            return true;
        if (!ssd)
            ssd = this.getSetupStepDescription(modelId);
        if (ssd.check) {
            const check = await ssd.check();
            if (check)
                await this.update({ complete: true });
            return check;
        }
        else {
            return null;
        }
    }
    async getOutcome(ssd, modelId) {
        if (!ssd)
            ssd = this.getSetupStepDescription(modelId);
        if (ssd.outcome)
            return ssd.outcome();
        return null;
    }
    async getDisabled(ssd, modelId) {
        if (!ssd)
            ssd = this.getSetupStepDescription(modelId);
        return ssd.disabled();
    }
    getSetupStepDescription(modelId) {
        const setupSteps = (0, setupSteps_1.getSetupStepDescriptions)(modelId);
        const ssdMatch = setupSteps.find((ssd) => ssd.key === this.key);
        if (ssdMatch)
            return ssdMatch;
        throw new Error(`Cannot find Setup Step Description for key ${this.key}`);
    }
};
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.Column,
    __metadata("design:type", Number)
], SetupStep.prototype, "position", void 0);
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    (0, sequelize_typescript_1.Length)({ min: 3, max: 191 }),
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], SetupStep.prototype, "key", void 0);
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    (0, sequelize_typescript_1.Default)(false),
    sequelize_typescript_1.Column,
    __metadata("design:type", Boolean)
], SetupStep.prototype, "skipped", void 0);
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    (0, sequelize_typescript_1.Default)(false),
    sequelize_typescript_1.Column,
    __metadata("design:type", Boolean)
], SetupStep.prototype, "complete", void 0);
SetupStep = __decorate([
    (0, sequelize_typescript_1.Table)({ tableName: "setupSteps", paranoid: false })
], SetupStep);
exports.SetupStep = SetupStep;
