"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TeamMember = void 0;
const sequelize_typescript_1 = require("sequelize-typescript");
const Team_1 = require("./Team");
const teamMember_1 = require("../modules/ops/teamMember");
const lockableHelper_1 = require("../modules/lockableHelper");
const apiData_1 = require("../modules/apiData");
const cls_1 = require("../modules/cls");
const commonModel_1 = require("../classes/commonModel");
let TeamMember = class TeamMember extends commonModel_1.CommonModel {
    idPrefix() {
        return "tem";
    }
    async apiData() {
        return {
            id: this.id,
            teamId: this.teamId,
            firstName: this.firstName,
            lastName: this.lastName,
            email: this.email,
            locked: this.locked,
            lastLoginAt: this.lastLoginAt,
            createdAt: apiData_1.APIData.formatDate(this.createdAt),
            updatedAt: apiData_1.APIData.formatDate(this.updatedAt),
        };
    }
    async updatePassword(password) {
        return teamMember_1.TeamMemberOps.updatePassword(this, password);
    }
    async checkPassword(password) {
        return teamMember_1.TeamMemberOps.checkPassword(this, password);
    }
    // --- Class Methods --- //
    static lowercaseEmail(instance) {
        if (instance.email)
            instance.email = instance.email.toLocaleLowerCase();
    }
    static async ensureTeamExists(instance) {
        const team = await instance.$get("team");
        if (!team)
            throw new Error("team not found");
    }
    static async noUpdateIfLocked(instance) {
        await lockableHelper_1.LockableHelper.beforeSave(instance, ["lastLoginAt"]);
    }
    static async sendTelemetry() {
        await cls_1.CLS.enqueueTask("telemetry:adHoc", { trigger: "team" });
    }
    static async noDestroyIfLocked(instance) {
        await lockableHelper_1.LockableHelper.beforeDestroy(instance);
    }
};
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.Column,
    (0, sequelize_typescript_1.ForeignKey)(() => Team_1.Team),
    __metadata("design:type", String)
], TeamMember.prototype, "teamId", void 0);
__decorate([
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], TeamMember.prototype, "locked", void 0);
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], TeamMember.prototype, "firstName", void 0);
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], TeamMember.prototype, "lastName", void 0);
__decorate([
    (0, sequelize_typescript_1.AllowNull)(false),
    sequelize_typescript_1.IsEmail,
    sequelize_typescript_1.IsLowercase,
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], TeamMember.prototype, "email", void 0);
__decorate([
    sequelize_typescript_1.Column,
    __metadata("design:type", String)
], TeamMember.prototype, "passwordHash", void 0);
__decorate([
    sequelize_typescript_1.Column,
    __metadata("design:type", Date)
], TeamMember.prototype, "lastLoginAt", void 0);
__decorate([
    (0, sequelize_typescript_1.BelongsTo)(() => Team_1.Team),
    __metadata("design:type", Team_1.Team)
], TeamMember.prototype, "team", void 0);
__decorate([
    sequelize_typescript_1.BeforeValidate,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [TeamMember]),
    __metadata("design:returntype", void 0)
], TeamMember, "lowercaseEmail", null);
__decorate([
    sequelize_typescript_1.BeforeSave,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [TeamMember]),
    __metadata("design:returntype", Promise)
], TeamMember, "ensureTeamExists", null);
__decorate([
    sequelize_typescript_1.BeforeSave,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [TeamMember]),
    __metadata("design:returntype", Promise)
], TeamMember, "noUpdateIfLocked", null);
__decorate([
    sequelize_typescript_1.AfterCreate,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", Promise)
], TeamMember, "sendTelemetry", null);
__decorate([
    sequelize_typescript_1.BeforeDestroy,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [TeamMember]),
    __metadata("design:returntype", Promise)
], TeamMember, "noDestroyIfLocked", null);
TeamMember = __decorate([
    (0, sequelize_typescript_1.Table)({ tableName: "teamMembers", paranoid: false })
], TeamMember);
exports.TeamMember = TeamMember;
