"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DestinationsCache = void 0;
const modelCache_1 = require("../modelCache");
const Destination_1 = require("../../models/Destination");
const Option_1 = require("../../models/Option");
const Mapping_1 = require("../../models/Mapping");
async function findAllWithCache(modelId, state) {
    const now = Date.now();
    if (this.expires > now && this.instances.length > 0) {
        return modelId
            ? this.instances.filter((s) => (s === null || s === void 0 ? void 0 : s.modelId) === modelId && (!state || s.state === state))
            : this.instances.filter((s) => !state || s.state === state);
    }
    else {
        this.instances = await Destination_1.Destination.unscoped().findAll({
            include: this.include(),
        });
        this.expires = now + this.TTL;
        return modelId
            ? this.instances.filter((s) => (s === null || s === void 0 ? void 0 : s.modelId) === modelId && (!state || s.state === state))
            : this.instances.filter((s) => !state || s.state === state);
    }
}
async function findOneWithCache(value, modelId, state, lookupKey = "id") {
    const instances = await this.findAllWithCache(modelId);
    let instance = instances.find((i) => i[lookupKey] === value);
    if (!instance) {
        const where = { [lookupKey]: value };
        if (state)
            where.state = state;
        instance = await Destination_1.Destination.unscoped().findOne({
            where,
            include: this.include(),
        });
        if (instance)
            this.invalidate();
    }
    return instance;
}
exports.DestinationsCache = new modelCache_1.ModelCache(findAllWithCache, findOneWithCache, () => [
    { model: Option_1.Option, required: false },
    { model: Mapping_1.Mapping, required: false },
]);
