import FormData from "form-data";
export declare class CloudError extends Error {
    code: string;
    constructor({ code, message }: {
        code: string;
        message: string;
    });
}
export declare function packageConfig(projectPath: string, configDirPath: string, tarballPath: string): Promise<string>;
export interface ConfigurationApiData {
    id: string;
    state: string;
    projectId: string;
    toApply: boolean;
    errorMessage?: string;
    applyJobId?: string;
    validateJobId?: string;
    coreVersion: string;
    processedAt: string;
    validatedAt: string;
    appliedAt: string;
    finishedAt: string;
    createdAt: string;
    updatedAt: string;
}
export interface JobApiData {
    id: string;
    type: string;
    state: string;
    configurationId: string;
    logs: string;
    completedAt: string;
    createdAt: string;
    updatedAt: string;
}
export declare class CloudClient {
    baseUrl: string;
    token: string;
    projectId: string;
    constructor(projectId: string, token: string);
    request<T>(url: string, options?: RequestInit & {
        _buildFormData?: () => FormData;
    }, attempts?: number): Promise<T>;
    createConfiguration(tarballPath: string, toApply: boolean, message?: string, externalUrl?: string): Promise<ConfigurationApiData>;
    getConfiguration(configurationId: string): Promise<ConfigurationApiData>;
    getJob(jobId: string): Promise<JobApiData>;
}
