"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteApiKeys = exports.loadApiKey = void 0;
const codeConfig_1 = require("../../classes/codeConfig");
const __1 = require("../.."); // configLoader imports need to be from root
const sequelize_1 = require("sequelize");
async function loadApiKey(configObject, externallyValidate, validate = false) {
    let isNew = false;
    (0, codeConfig_1.validateConfigObjectKeys)(__1.ApiKey, configObject, ["options", "permissions"]);
    let apiKey = await __1.ApiKey.scope(null).findOne({
        where: { locked: (0, codeConfig_1.getCodeConfigLockKey)(), id: configObject.id },
    });
    if (!apiKey) {
        isNew = true;
        apiKey = await __1.ApiKey.create({
            id: configObject.id,
            locked: (0, codeConfig_1.getCodeConfigLockKey)(),
            name: configObject.name,
        });
    }
    await apiKey.update({ apiKey: configObject.apiKey, name: configObject.name });
    if (configObject.options) {
        await apiKey.update({
            permissionAllRead: configObject.options.permissionAllRead,
            permissionAllWrite: configObject.options.permissionAllWrite,
        });
    }
    if (configObject.permissions) {
        await apiKey.setPermissions(configObject.permissions);
    }
    await __1.Permission.update({ locked: (0, codeConfig_1.getCodeConfigLockKey)() }, { where: { ownerId: apiKey.id } });
    (0, codeConfig_1.logModel)(apiKey, validate ? "validated" : isNew ? "created" : "updated");
    return { apikey: [apiKey.id] };
}
exports.loadApiKey = loadApiKey;
async function deleteApiKeys(ids) {
    const apiKeys = await __1.ApiKey.scope(null).findAll({
        where: { locked: (0, codeConfig_1.getCodeConfigLockKey)(), id: { [sequelize_1.Op.notIn]: ids } },
    });
    for (const i in apiKeys) {
        await apiKeys[i].destroy();
        (0, codeConfig_1.logModel)(apiKeys[i], "deleted");
    }
    return apiKeys.map((instance) => instance.id);
}
exports.deleteApiKeys = deleteApiKeys;
