"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteModels = exports.loadModel = void 0;
const codeConfig_1 = require("../../classes/codeConfig");
const __1 = require("../.."); // configLoader imports need to be from root
const sequelize_1 = require("sequelize");
const configWriter_1 = require("../configWriter");
async function loadModel(configObject, externallyValidate, validate = false) {
    let isNew = false;
    (0, codeConfig_1.validateConfigObjectKeys)(__1.GrouparooModel, configObject);
    let model = await __1.GrouparooModel.scope(null).findOne({
        where: {
            id: configObject.id,
            locked: { [sequelize_1.Op.or]: [(0, codeConfig_1.getCodeConfigLockKey)(), null] }, // allow the default model to be modified
        },
    });
    if (!model) {
        isNew = true;
        model = await __1.GrouparooModel.create({
            id: configObject.id,
            name: configObject.name,
            type: configObject.type,
        });
    }
    await model.update({
        type: configObject.type,
        name: configObject.name,
        locked: configWriter_1.ConfigWriter.getLockKey(configObject),
    });
    (0, codeConfig_1.logModel)(model, validate ? "validated" : isNew ? "created" : "updated");
    return { model: [model.id] };
}
exports.loadModel = loadModel;
async function deleteModels(ids) {
    const models = await __1.GrouparooModel.scope(null).findAll({
        where: {
            locked: (0, codeConfig_1.getCodeConfigLockKey)(),
            id: { [sequelize_1.Op.notIn]: ids },
        },
    });
    for (const i in models) {
        await models[i].update({ state: "deleted", locked: null });
        (0, codeConfig_1.logModel)(models[i], "deleted");
    }
    return models.map((instance) => instance.id);
}
exports.deleteModels = deleteModels;
