/**
 * List of conversions from Grouparoo Types to Destination Types
 * In many cases, we can do conversions, ie: 'integer' or 'float' to 'number' or there are cast-able representations in other types, like 'integer' to 'string'.
 * Any conversion that's not listed here is not valid.
 * Learn more at https://docs.google.com/spreadsheets/d/1Fbkdsq_IR8deOYF4QpVq_XIdqpvAah3tJbVP2K5nCdc
 */
export declare const destinationTypeConversions: {
    float: {
        any: (v: number) => number;
        float: (v: number) => number;
        string: (v: number) => string;
        number: (v: number) => number;
    };
    integer: {
        any: (v: number) => number;
        float: (v: number) => number;
        integer: (v: number) => number;
        string: (v: number) => string;
        number: (v: number) => number;
    };
    string: {
        any: (v: string) => string;
        string: (v: string) => string;
        url: (v: string) => string;
        email: (v: string) => string;
        boolean: (v: string) => string;
        date: (v: string) => string;
        float: (v: string) => string;
        integer: (v: string) => string;
        number: (v: string) => string;
        phoneNumber: (v: string) => string;
    };
    url: {
        any: (v: string) => string;
        string: (v: string) => string;
        url: (v: string) => string;
    };
    email: {
        any: (v: string) => string;
        string: (v: string) => string;
        email: (v: string) => string;
    };
    phoneNumber: {
        any: (v: string) => string;
        string: (v: string) => string;
        phoneNumber: (v: string) => string;
    };
    boolean: {
        any: (v: boolean) => boolean;
        string: (v: boolean) => string;
        boolean: (v: boolean) => boolean;
        number: (v: boolean) => 1 | 0;
    };
    date: {
        any: (v: Date) => Date;
        float: (v: Date) => number;
        integer: (v: Date) => number;
        string: (v: Date) => string;
        date: (v: Date) => Date;
        number: (v: Date) => number;
    };
};
