"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.destinationTypeConversions = void 0;
/**
 * List of conversions from Grouparoo Types to Destination Types
 * In many cases, we can do conversions, ie: 'integer' or 'float' to 'number' or there are cast-able representations in other types, like 'integer' to 'string'.
 * Any conversion that's not listed here is not valid.
 * Learn more at https://docs.google.com/spreadsheets/d/1Fbkdsq_IR8deOYF4QpVq_XIdqpvAah3tJbVP2K5nCdc
 */
exports.destinationTypeConversions = {
    float: {
        any: (v) => v,
        float: (v) => v,
        string: (v) => v.toString(),
        number: (v) => v,
    },
    integer: {
        any: (v) => v,
        float: (v) => v,
        integer: (v) => v,
        string: (v) => v.toString(),
        number: (v) => v,
    },
    string: {
        any: (v) => v,
        string: (v) => v,
        url: (v) => v,
        email: (v) => v,
        boolean: (v) => v,
        date: (v) => v,
        float: (v) => v,
        integer: (v) => v,
        number: (v) => v,
        phoneNumber: (v) => v,
    },
    url: {
        any: (v) => v,
        string: (v) => v,
        url: (v) => v,
    },
    email: {
        any: (v) => v,
        string: (v) => v,
        email: (v) => v,
    },
    phoneNumber: {
        any: (v) => v,
        string: (v) => v,
        phoneNumber: (v) => v,
    },
    boolean: {
        any: (v) => v,
        string: (v) => v.toString(),
        boolean: (v) => v,
        number: (v) => (v === true ? 1 : 0),
    },
    date: {
        any: (v) => v,
        float: (v) => v.getTime(),
        integer: (v) => v.getTime(),
        string: (v) => v.toISOString(),
        date: (v) => v,
        number: (v) => v.getTime(),
    },
};
