import { Property } from "../models/Property";
import { Schedule } from "../models/Schedule";
import { FilterOpsDescriptions } from "./filterOpsDictionary";
export declare namespace FilterHelper {
    interface FiltersWithKey {
        key: string;
        op: keyof typeof FilterOpsDescriptions;
        match?: string | number | boolean;
        relativeMatchNumber?: number;
        relativeMatchUnit?: string;
        relativeMatchDirection?: string;
    }
    const deprecatedFilters: {
        equals: string;
        "does not equal": string;
        "greater than": string;
        "greater than or equal to": string;
        "less than": string;
        "less than or equal to": string;
        contains: string;
        "does not contain": string;
    };
    /**
     * Get the existing filters for an instance
     *
     */
    function getFilters(instance: Property | Schedule): Promise<FiltersWithKey[]>;
    /**
     * Set the filters for an instance
     *
     */
    function setFilters(instance: Property | Schedule, filters: FiltersWithKey[], externallyValidate?: boolean): Promise<void>;
    /**
     * Validate filters for an instance
     *
     */
    function validateFilters(instance: Property | Schedule, filters: FiltersWithKey[]): Promise<void>;
    /**
     * Get the options for an instance's Filter from its plugin
     */
    function pluginFilterOptions(instance: Property | Schedule): Promise<import("..").SourceFilterMethodResponseRow[]>;
    /**
     * Determine if these Filters are equal
     */
    function filtersAreEqual(oldFilters: FiltersWithKey[], newFilters: FiltersWithKey[]): boolean;
}
