"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GrouparooSubscription = void 0;
const cls_1 = require("../modules/cls");
const plugin_1 = require("../modules/plugin");
const actionhero_1 = require("actionhero");
const path_1 = __importDefault(require("path"));
require("isomorphic-fetch"); // I need to be required vs imported to avoid TS conflicts with the @grouparoo/ui-* package which has its own fetch polyfill
const packageJSON = require(path_1.default.join(__dirname, "..", "..", "package.json"));
const route = "/api/v1/subscribers";
const source = `${packageJSON.name}`;
const medium = "app";
const campaign = `v${packageJSON.version}`;
/**
 * We want this run "in-thread" but not to be blocking.  We cannot rely on tasks, as the user may not be running them.
 * We also want to allow for this method to fail, and not block anything else in the server.
 * Therefore, we don't await the top-level CLS.afterCommit method
 */
async function GrouparooSubscription({ teamMember, email: subscriberEmail, subscribed = true, }) {
    if (actionhero_1.env === "test")
        return;
    if (!actionhero_1.config.telemetry.enabled)
        return;
    if (!teamMember && !subscriberEmail)
        return;
    const email = (teamMember === null || teamMember === void 0 ? void 0 : teamMember.email) || subscriberEmail;
    cls_1.CLS.afterCommit(async () => {
        const setting = await plugin_1.plugin.readSetting("telemetry", "customer-id");
        const customerId = setting.value;
        try {
            const response = await fetch(`${actionhero_1.config.telemetry.host}${route}`, {
                method: "POST",
                headers: { "Content-Type": "application/json" },
                body: JSON.stringify({
                    email,
                    firstName: teamMember === null || teamMember === void 0 ? void 0 : teamMember.firstName,
                    lastName: teamMember === null || teamMember === void 0 ? void 0 : teamMember.lastName,
                    source,
                    medium,
                    campaign,
                    customerId,
                    subscribed,
                }),
            }).then((r) => r.json());
            if (response.error)
                throw response.error;
            (0, actionhero_1.log)(`Registered ${email} for Grouparoo subscription to ${customerId}`);
        }
        catch (error) {
            (0, actionhero_1.log)(`Error subscribing to Grouparoo Subscription: ${error}`, "error");
        }
    });
}
exports.GrouparooSubscription = GrouparooSubscription;
