"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LockableHelper = void 0;
const actionhero_1 = require("actionhero");
const modelName_1 = require("./modelName");
var LockableHelper;
(function (LockableHelper) {
    async function beforeSave(instance, allowedColumnsThatCanChangeWhenLocked = []) {
        var _a;
        // because of the strange way that SQLite handles undefined values, we need to convert any `undefined` to `null`
        if (instance.locked === undefined)
            instance.locked = null;
        if (!Boolean(instance.locked))
            return;
        if (((_a = actionhero_1.api === null || actionhero_1.api === void 0 ? void 0 : actionhero_1.api.codeConfig) === null || _a === void 0 ? void 0 : _a.allowLockedModelChanges) !== false)
            return;
        if (instance.isNewRecord)
            return;
        allowedColumnsThatCanChangeWhenLocked.push("locked", "updatedAt", "createdAt");
        const changedCols = instance.changed(); // ['firstName', 'lastLoginAt']
        if (Array.isArray(changedCols)) {
            for (const i in changedCols) {
                if (!allowedColumnsThatCanChangeWhenLocked.includes(changedCols[i])) {
                    throw new Error(`you cannot update this locked ${(0, modelName_1.modelName)(instance)} (${instance.id} - ${instance.locked}) [${changedCols
                        .map((k) => `\`${k}\``)
                        .join(", ")} has changes]`);
                }
            }
        }
    }
    LockableHelper.beforeSave = beforeSave;
    async function beforeUpdateOptions(instance, hasChanges = true) {
        var _a;
        if (!Boolean(instance.locked))
            return;
        if (((_a = actionhero_1.api === null || actionhero_1.api === void 0 ? void 0 : actionhero_1.api.codeConfig) === null || _a === void 0 ? void 0 : _a.allowLockedModelChanges) !== false)
            return;
        if (hasChanges) {
            throw new Error(`you cannot update the options for a locked ${(0, modelName_1.modelName)(instance)} (${instance.id}) - ${instance.locked}`);
        }
    }
    LockableHelper.beforeUpdateOptions = beforeUpdateOptions;
    async function beforeDestroy(instance) {
        var _a;
        if (!Boolean(instance.locked))
            return;
        if (((_a = actionhero_1.api === null || actionhero_1.api === void 0 ? void 0 : actionhero_1.api.codeConfig) === null || _a === void 0 ? void 0 : _a.allowLockedModelChanges) !== false)
            return;
        throw new Error(`you cannot destroy a locked ${(0, modelName_1.modelName)(instance)} (${instance.id} - ${instance.locked})`);
    }
    LockableHelper.beforeDestroy = beforeDestroy;
})(LockableHelper = exports.LockableHelper || (exports.LockableHelper = {}));
