"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelGuard = void 0;
const modelsCache_1 = require("./caches/modelsCache");
var ModelGuard;
(function (ModelGuard) {
    async function check(instance) {
        if (instance.isNewRecord) {
            const model = await modelsCache_1.ModelsCache.findOneWithCache(instance.modelId);
            if (!model) {
                throw new Error(`cannot find model with id "${instance.modelId}"`);
            }
            if (instance.state !== "deleted" && model.state === "deleted") {
                throw new Error(`cannot find ready model with id "${instance.modelId}"`);
            }
        }
        else {
            // we are updating an existing instance
            const changes = instance.changed();
            if (!changes)
                return;
            if (changes.includes("modelId")) {
                throw new Error("cannot change models");
            }
        }
    }
    ModelGuard.check = check;
})(ModelGuard = exports.ModelGuard || (exports.ModelGuard = {}));
