import { Run } from "../../models/Run";
import { Group } from "../../models/Group";
import { Schedule } from "../../models/Schedule";
import { GrouparooModel } from "../../models/GrouparooModel";
import { Property } from "../../models/Property";
export declare namespace RunOps {
    /**
     * Create a Run for this Group or Model
     */
    function run(creator: Group | GrouparooModel, destinationId?: string): Promise<Run>;
    /**
     * Stop previous Runs for this Group
     */
    function stopPreviousRuns(creator: Group | GrouparooModel): Promise<void>;
    function getCreatorTypeString(creator: Schedule | Property | Group | GrouparooModel): "group" | "schedule" | "property" | "grouparooModel";
    /**
     * Return counts of the states of each import in N chunks over the lifetime of the run
     * Great for drawing charts!
     */
    function quantizedTimeline(run: Run, steps?: number): Promise<{
        lastBoundary: number;
        nextBoundary: number;
        steps: {
            associate: number;
            imported: number;
        };
    }[]>;
    /**
     * Count up the totals from imports for `importsCreated`, `recordsCreated` and `recordsImported`
     */
    function updateTotals(run: Run): Promise<void>;
    /**
     * Make a guess to what percent complete this Run is
     */
    function determinePercentComplete(run: Run): Promise<number>;
    /**
     * Is there already a task enqueued to process this run?
     */
    function isRunEnqueued(taskName: string, runId: string): Promise<boolean>;
}
