"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FinalSummary = exports.Status = void 0;
const actionhero_1 = require("actionhero");
const Group_1 = require("../models/Group");
const GrouparooRecord_1 = require("../models/GrouparooRecord");
const statusReporters_1 = require("./statusReporters");
const uuid = __importStar(require("uuid"));
var Status;
(function (Status) {
    Status.maxSamples = 50;
    Status.sampleFrequencyMS = 1000 * 5;
    Status.cachePrefix = `${actionhero_1.config.general.cachePrefix}status:samples:`;
    Status.statusSampleReporters = [
        // information about how Grouparoo is being operated
        async () => statusReporters_1.StatusReporters.Cluster.Workers.countWorkers(),
        async () => statusReporters_1.StatusReporters.Cluster.Workers.countErrors(),
        async () => statusReporters_1.StatusReporters.Cluster.Workers.details(),
        async () => statusReporters_1.StatusReporters.Cluster.Process.platform(),
        async () => statusReporters_1.StatusReporters.Cluster.Process.env(),
        async () => statusReporters_1.StatusReporters.Cluster.Notifications.unread(),
        // usage counts (we only care about some of the models)
        async () => statusReporters_1.StatusReporters.Totals.Models([GrouparooRecord_1.GrouparooRecord, Group_1.Group]),
        async () => statusReporters_1.StatusReporters.Totals.UniqueRecordsExported(),
        // thing in progress
        async () => statusReporters_1.StatusReporters.Pending.pendingImports(),
        async () => statusReporters_1.StatusReporters.Pending.pendingImportsBySource(),
        async () => statusReporters_1.StatusReporters.Pending.pendingExports(),
        async () => statusReporters_1.StatusReporters.Pending.pendingExportsByDestination(),
        async () => statusReporters_1.StatusReporters.Pending.pendingRecords(),
        async () => statusReporters_1.StatusReporters.Pending.pendingRuns(),
        //things waiting to be deleted
        async () => statusReporters_1.StatusReporters.Deleted.deletedDestinations(),
        async () => statusReporters_1.StatusReporters.Deleted.deletedGroups(),
        async () => statusReporters_1.StatusReporters.Deleted.deletedProperties(),
        async () => statusReporters_1.StatusReporters.Deleted.deletedSources(),
        async () => statusReporters_1.StatusReporters.Deleted.deletedRecords(),
        async () => statusReporters_1.StatusReporters.Deleted.deletedModels(),
        // additional things
        async () => statusReporters_1.StatusReporters.Groups.byNewestMember(),
        async () => statusReporters_1.StatusReporters.Sources.nextRuns(),
    ];
    async function get(limit = Status.maxSamples) {
        const redis = getRedis();
        const keyMatch = `${Status.cachePrefix}*`;
        const keys = await redis.keys(keyMatch);
        if (keys.length === 0)
            return {};
        const samples = (await redis.mget(keys))
            .map((v) => {
            let parsed;
            try {
                parsed = JSON.parse(v);
            }
            catch { }
            return parsed;
        })
            .filter((sample) => sample &&
            sample.timestamp &&
            sample.metric &&
            sample.metric.topic &&
            sample.metric.collection);
        const response = {};
        samples.forEach((sample) => {
            if (!response[sample.metric.topic])
                response[sample.metric.topic] = {};
            if (!response[sample.metric.topic][sample.metric.collection])
                response[sample.metric.topic][sample.metric.collection] = [];
            response[sample.metric.topic][sample.metric.collection].push(sample);
            response[sample.metric.topic][sample.metric.collection] = response[sample.metric.topic][sample.metric.collection]
                .sort((a, b) => b.timestamp - a.timestamp)
                .slice(0, limit)
                .sort((a, b) => a.timestamp - b.timestamp);
        });
        return response;
    }
    Status.get = get;
    async function set(metrics, ttl = Math.ceil((Status.sampleFrequencyMS / 1000) * Status.maxSamples * 2)) {
        if (!Array.isArray(metrics))
            metrics = [metrics];
        const setMetrics = [];
        const now = new Date();
        const redis = getRedis();
        for (const metric of metrics) {
            const status = {
                timestamp: now.getTime(),
                metric,
            };
            const key = `${Status.cachePrefix}${now.getTime()}:${metric.topic}:${metric.collection}:${uuid.v4()}`;
            await redis.set(key, JSON.stringify(status));
            await redis.expire(key, ttl);
            setMetrics.push(status);
        }
        return setMetrics;
    }
    Status.set = set;
    async function setAll() {
        for (const method of Status.statusSampleReporters) {
            const response = await method();
            const metrics = await Status.set(response);
            await actionhero_1.chatRoom.broadcast({}, "system:status", { metrics });
        }
    }
    Status.setAll = setAll;
    function getRedis() {
        return actionhero_1.api.redis.clients.client;
    }
    async function getCurrent() {
        return Status.get(1); // 1 => current sample
    }
    Status.getCurrent = getCurrent;
})(Status = exports.Status || (exports.Status = {}));
var FinalSummary;
(function (FinalSummary) {
    async function getFinalSummary() {
        const finalSummaryLogs = [];
        finalSummaryLogs.push(await statusReporters_1.FinalSummaryReporters.GrouparooRecords.getData());
        finalSummaryLogs.push(await statusReporters_1.FinalSummaryReporters.Sources.getData());
        finalSummaryLogs.push(await statusReporters_1.FinalSummaryReporters.Destinations.getData());
        finalSummaryLogs.push(await statusReporters_1.FinalSummaryReporters.Warnings.getWarnings());
        return finalSummaryLogs;
    }
    FinalSummary.getFinalSummary = getFinalSummary;
})(FinalSummary = exports.FinalSummary || (exports.FinalSummary = {}));
