"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnqueueExportProcessors = void 0;
const actionhero_1 = require("actionhero");
const Destination_1 = require("../../models/Destination");
const retryableTask_1 = require("../../classes/tasks/retryableTask");
const plugin_1 = require("../../modules/plugin");
const exportProcessor_1 = require("../../modules/ops/exportProcessor");
class EnqueueExportProcessors extends retryableTask_1.RetryableTask {
    constructor() {
        super(...arguments);
        this.name = "export:enqueueProcessors";
        this.description = "check for pending export processors and enqueue other tasks to process them";
        this.frequency = 1000 * 30; // every 30 seconds
        this.queue = "exports";
        this.inputs = {};
    }
    async runWithinTransaction() {
        const delayMs = parseInt((await plugin_1.plugin.readSetting("core", "export-processors-retry-delay-seconds")).value) * 1000;
        let totalEnqueued = 0;
        const destinations = await Destination_1.Destination.scope(null).findAll();
        for (const i in destinations) {
            const enqueuedProcessorsCount = await exportProcessor_1.ExportProcessorOps.enqueuePendingProcessorsForDestination(destinations[i], delayMs);
            totalEnqueued += enqueuedProcessorsCount;
            if (enqueuedProcessorsCount > 0) {
                (0, actionhero_1.log)(`enqueued ${enqueuedProcessorsCount} export processors for ${destinations[i].name} (${destinations[i].id})`);
            }
        }
        return totalEnqueued;
    }
}
exports.EnqueueExportProcessors = EnqueueExportProcessors;
